"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
*!
Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2025)

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import streamlit.proto.GapSize_pb2
import streamlit.proto.HeightConfig_pb2
import streamlit.proto.WidthConfig_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Block(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class Vertical(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        BORDER_FIELD_NUMBER: builtins.int
        HEIGHT_FIELD_NUMBER: builtins.int
        border: builtins.bool
        height: builtins.int
        """DEPRECATED: We use height_config on the top-level which is shared.
        with other Block types.
        """
        def __init__(
            self,
            *,
            border: builtins.bool = ...,
            height: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["border", b"border", "height", b"height"]) -> None: ...

    @typing.final
    class Horizontal(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        GAP_FIELD_NUMBER: builtins.int
        gap: builtins.str
        def __init__(
            self,
            *,
            gap: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["gap", b"gap"]) -> None: ...

    @typing.final
    class FlexContainer(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _Direction:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _DirectionEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Block.FlexContainer._Direction.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            DIRECTION_UNDEFINED: Block.FlexContainer._Direction.ValueType  # 0
            VERTICAL: Block.FlexContainer._Direction.ValueType  # 1
            HORIZONTAL: Block.FlexContainer._Direction.ValueType  # 2

        class Direction(_Direction, metaclass=_DirectionEnumTypeWrapper): ...
        DIRECTION_UNDEFINED: Block.FlexContainer.Direction.ValueType  # 0
        VERTICAL: Block.FlexContainer.Direction.ValueType  # 1
        HORIZONTAL: Block.FlexContainer.Direction.ValueType  # 2

        class _Justify:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _JustifyEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Block.FlexContainer._Justify.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            JUSTIFY_UNDEFINED: Block.FlexContainer._Justify.ValueType  # 0
            JUSTIFY_START: Block.FlexContainer._Justify.ValueType  # 1
            JUSTIFY_END: Block.FlexContainer._Justify.ValueType  # 2
            JUSTIFY_CENTER: Block.FlexContainer._Justify.ValueType  # 3
            SPACE_BETWEEN: Block.FlexContainer._Justify.ValueType  # 4

        class Justify(_Justify, metaclass=_JustifyEnumTypeWrapper): ...
        JUSTIFY_UNDEFINED: Block.FlexContainer.Justify.ValueType  # 0
        JUSTIFY_START: Block.FlexContainer.Justify.ValueType  # 1
        JUSTIFY_END: Block.FlexContainer.Justify.ValueType  # 2
        JUSTIFY_CENTER: Block.FlexContainer.Justify.ValueType  # 3
        SPACE_BETWEEN: Block.FlexContainer.Justify.ValueType  # 4

        class _Align:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _AlignEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Block.FlexContainer._Align.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            ALIGN_UNDEFINED: Block.FlexContainer._Align.ValueType  # 0
            ALIGN_START: Block.FlexContainer._Align.ValueType  # 1
            ALIGN_END: Block.FlexContainer._Align.ValueType  # 2
            ALIGN_CENTER: Block.FlexContainer._Align.ValueType  # 3
            STRETCH: Block.FlexContainer._Align.ValueType  # 4

        class Align(_Align, metaclass=_AlignEnumTypeWrapper): ...
        ALIGN_UNDEFINED: Block.FlexContainer.Align.ValueType  # 0
        ALIGN_START: Block.FlexContainer.Align.ValueType  # 1
        ALIGN_END: Block.FlexContainer.Align.ValueType  # 2
        ALIGN_CENTER: Block.FlexContainer.Align.ValueType  # 3
        STRETCH: Block.FlexContainer.Align.ValueType  # 4

        BORDER_FIELD_NUMBER: builtins.int
        GAP_CONFIG_FIELD_NUMBER: builtins.int
        SCALE_FIELD_NUMBER: builtins.int
        DIRECTION_FIELD_NUMBER: builtins.int
        WRAP_FIELD_NUMBER: builtins.int
        JUSTIFY_FIELD_NUMBER: builtins.int
        ALIGN_FIELD_NUMBER: builtins.int
        border: builtins.bool
        scale: builtins.float
        direction: global___Block.FlexContainer.Direction.ValueType
        wrap: builtins.bool
        justify: global___Block.FlexContainer.Justify.ValueType
        align: global___Block.FlexContainer.Align.ValueType
        @property
        def gap_config(self) -> streamlit.proto.GapSize_pb2.GapConfig: ...
        def __init__(
            self,
            *,
            border: builtins.bool = ...,
            gap_config: streamlit.proto.GapSize_pb2.GapConfig | None = ...,
            scale: builtins.float = ...,
            direction: global___Block.FlexContainer.Direction.ValueType = ...,
            wrap: builtins.bool = ...,
            justify: global___Block.FlexContainer.Justify.ValueType = ...,
            align: global___Block.FlexContainer.Align.ValueType = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["gap_config", b"gap_config"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["align", b"align", "border", b"border", "direction", b"direction", "gap_config", b"gap_config", "justify", b"justify", "scale", b"scale", "wrap", b"wrap"]) -> None: ...

    @typing.final
    class Column(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _VerticalAlignment:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _VerticalAlignmentEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Block.Column._VerticalAlignment.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            TOP: Block.Column._VerticalAlignment.ValueType  # 0
            CENTER: Block.Column._VerticalAlignment.ValueType  # 1
            BOTTOM: Block.Column._VerticalAlignment.ValueType  # 2

        class VerticalAlignment(_VerticalAlignment, metaclass=_VerticalAlignmentEnumTypeWrapper): ...
        TOP: Block.Column.VerticalAlignment.ValueType  # 0
        CENTER: Block.Column.VerticalAlignment.ValueType  # 1
        BOTTOM: Block.Column.VerticalAlignment.ValueType  # 2

        WEIGHT_FIELD_NUMBER: builtins.int
        GAP_FIELD_NUMBER: builtins.int
        VERTICAL_ALIGNMENT_FIELD_NUMBER: builtins.int
        SHOW_BORDER_FIELD_NUMBER: builtins.int
        GAP_CONFIG_FIELD_NUMBER: builtins.int
        weight: builtins.float
        gap: builtins.str
        """Deprecated. Use gap_size"""
        vertical_alignment: global___Block.Column.VerticalAlignment.ValueType
        show_border: builtins.bool
        @property
        def gap_config(self) -> streamlit.proto.GapSize_pb2.GapConfig: ...
        def __init__(
            self,
            *,
            weight: builtins.float = ...,
            gap: builtins.str = ...,
            vertical_alignment: global___Block.Column.VerticalAlignment.ValueType = ...,
            show_border: builtins.bool = ...,
            gap_config: streamlit.proto.GapSize_pb2.GapConfig | None = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["_gap_config", b"_gap_config", "gap_config", b"gap_config"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["_gap_config", b"_gap_config", "gap", b"gap", "gap_config", b"gap_config", "show_border", b"show_border", "vertical_alignment", b"vertical_alignment", "weight", b"weight"]) -> None: ...
        def WhichOneof(self, oneof_group: typing.Literal["_gap_config", b"_gap_config"]) -> typing.Literal["gap_config"] | None: ...

    @typing.final
    class Expandable(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LABEL_FIELD_NUMBER: builtins.int
        EXPANDED_FIELD_NUMBER: builtins.int
        ICON_FIELD_NUMBER: builtins.int
        label: builtins.str
        expanded: builtins.bool
        icon: builtins.str
        def __init__(
            self,
            *,
            label: builtins.str = ...,
            expanded: builtins.bool | None = ...,
            icon: builtins.str = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["_expanded", b"_expanded", "expanded", b"expanded"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["_expanded", b"_expanded", "expanded", b"expanded", "icon", b"icon", "label", b"label"]) -> None: ...
        def WhichOneof(self, oneof_group: typing.Literal["_expanded", b"_expanded"]) -> typing.Literal["expanded"] | None: ...

    @typing.final
    class Dialog(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _DialogWidth:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _DialogWidthEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Block.Dialog._DialogWidth.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            SMALL: Block.Dialog._DialogWidth.ValueType  # 0
            LARGE: Block.Dialog._DialogWidth.ValueType  # 1
            MEDIUM: Block.Dialog._DialogWidth.ValueType  # 2

        class DialogWidth(_DialogWidth, metaclass=_DialogWidthEnumTypeWrapper): ...
        SMALL: Block.Dialog.DialogWidth.ValueType  # 0
        LARGE: Block.Dialog.DialogWidth.ValueType  # 1
        MEDIUM: Block.Dialog.DialogWidth.ValueType  # 2

        TITLE_FIELD_NUMBER: builtins.int
        DISMISSIBLE_FIELD_NUMBER: builtins.int
        WIDTH_FIELD_NUMBER: builtins.int
        IS_OPEN_FIELD_NUMBER: builtins.int
        ID_FIELD_NUMBER: builtins.int
        title: builtins.str
        dismissible: builtins.bool
        width: global___Block.Dialog.DialogWidth.ValueType
        is_open: builtins.bool
        id: builtins.str
        """If id is set, dismissal of the dialog will trigger a rerun."""
        def __init__(
            self,
            *,
            title: builtins.str = ...,
            dismissible: builtins.bool = ...,
            width: global___Block.Dialog.DialogWidth.ValueType = ...,
            is_open: builtins.bool | None = ...,
            id: builtins.str = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["_is_open", b"_is_open", "is_open", b"is_open"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["_is_open", b"_is_open", "dismissible", b"dismissible", "id", b"id", "is_open", b"is_open", "title", b"title", "width", b"width"]) -> None: ...
        def WhichOneof(self, oneof_group: typing.Literal["_is_open", b"_is_open"]) -> typing.Literal["is_open"] | None: ...

    @typing.final
    class Form(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        FORM_ID_FIELD_NUMBER: builtins.int
        CLEAR_ON_SUBMIT_FIELD_NUMBER: builtins.int
        BORDER_FIELD_NUMBER: builtins.int
        ENTER_TO_SUBMIT_FIELD_NUMBER: builtins.int
        form_id: builtins.str
        clear_on_submit: builtins.bool
        border: builtins.bool
        enter_to_submit: builtins.bool
        def __init__(
            self,
            *,
            form_id: builtins.str = ...,
            clear_on_submit: builtins.bool = ...,
            border: builtins.bool = ...,
            enter_to_submit: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["border", b"border", "clear_on_submit", b"clear_on_submit", "enter_to_submit", b"enter_to_submit", "form_id", b"form_id"]) -> None: ...

    @typing.final
    class TabContainer(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        DEFAULT_TAB_INDEX_FIELD_NUMBER: builtins.int
        default_tab_index: builtins.int
        def __init__(
            self,
            *,
            default_tab_index: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["default_tab_index", b"default_tab_index"]) -> None: ...

    @typing.final
    class Tab(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LABEL_FIELD_NUMBER: builtins.int
        label: builtins.str
        def __init__(
            self,
            *,
            label: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["label", b"label"]) -> None: ...

    @typing.final
    class Popover(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        LABEL_FIELD_NUMBER: builtins.int
        USE_CONTAINER_WIDTH_FIELD_NUMBER: builtins.int
        HELP_FIELD_NUMBER: builtins.int
        DISABLED_FIELD_NUMBER: builtins.int
        ICON_FIELD_NUMBER: builtins.int
        TYPE_FIELD_NUMBER: builtins.int
        label: builtins.str
        use_container_width: builtins.bool
        """DEPRECATED: We use widthConfig on the top-level which is shared."""
        help: builtins.str
        disabled: builtins.bool
        icon: builtins.str
        type: builtins.str
        def __init__(
            self,
            *,
            label: builtins.str = ...,
            use_container_width: builtins.bool = ...,
            help: builtins.str = ...,
            disabled: builtins.bool = ...,
            icon: builtins.str = ...,
            type: builtins.str = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["disabled", b"disabled", "help", b"help", "icon", b"icon", "label", b"label", "type", b"type", "use_container_width", b"use_container_width"]) -> None: ...

    @typing.final
    class ChatMessage(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _AvatarType:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _AvatarTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Block.ChatMessage._AvatarType.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            IMAGE: Block.ChatMessage._AvatarType.ValueType  # 0
            EMOJI: Block.ChatMessage._AvatarType.ValueType  # 1
            ICON: Block.ChatMessage._AvatarType.ValueType  # 2

        class AvatarType(_AvatarType, metaclass=_AvatarTypeEnumTypeWrapper): ...
        IMAGE: Block.ChatMessage.AvatarType.ValueType  # 0
        EMOJI: Block.ChatMessage.AvatarType.ValueType  # 1
        ICON: Block.ChatMessage.AvatarType.ValueType  # 2

        NAME_FIELD_NUMBER: builtins.int
        AVATAR_FIELD_NUMBER: builtins.int
        AVATAR_TYPE_FIELD_NUMBER: builtins.int
        name: builtins.str
        avatar: builtins.str
        avatar_type: global___Block.ChatMessage.AvatarType.ValueType
        def __init__(
            self,
            *,
            name: builtins.str = ...,
            avatar: builtins.str = ...,
            avatar_type: global___Block.ChatMessage.AvatarType.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["avatar", b"avatar", "avatar_type", b"avatar_type", "name", b"name"]) -> None: ...

    VERTICAL_FIELD_NUMBER: builtins.int
    HORIZONTAL_FIELD_NUMBER: builtins.int
    COLUMN_FIELD_NUMBER: builtins.int
    EXPANDABLE_FIELD_NUMBER: builtins.int
    FORM_FIELD_NUMBER: builtins.int
    TAB_CONTAINER_FIELD_NUMBER: builtins.int
    TAB_FIELD_NUMBER: builtins.int
    CHAT_MESSAGE_FIELD_NUMBER: builtins.int
    POPOVER_FIELD_NUMBER: builtins.int
    DIALOG_FIELD_NUMBER: builtins.int
    FLEX_CONTAINER_FIELD_NUMBER: builtins.int
    ALLOW_EMPTY_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    HEIGHT_CONFIG_FIELD_NUMBER: builtins.int
    WIDTH_CONFIG_FIELD_NUMBER: builtins.int
    allow_empty: builtins.bool
    id: builtins.str
    @property
    def vertical(self) -> global___Block.Vertical: ...
    @property
    def horizontal(self) -> global___Block.Horizontal: ...
    @property
    def column(self) -> global___Block.Column: ...
    @property
    def expandable(self) -> global___Block.Expandable: ...
    @property
    def form(self) -> global___Block.Form: ...
    @property
    def tab_container(self) -> global___Block.TabContainer: ...
    @property
    def tab(self) -> global___Block.Tab: ...
    @property
    def chat_message(self) -> global___Block.ChatMessage: ...
    @property
    def popover(self) -> global___Block.Popover: ...
    @property
    def dialog(self) -> global___Block.Dialog: ...
    @property
    def flex_container(self) -> global___Block.FlexContainer: ...
    @property
    def height_config(self) -> streamlit.proto.HeightConfig_pb2.HeightConfig: ...
    @property
    def width_config(self) -> streamlit.proto.WidthConfig_pb2.WidthConfig: ...
    def __init__(
        self,
        *,
        vertical: global___Block.Vertical | None = ...,
        horizontal: global___Block.Horizontal | None = ...,
        column: global___Block.Column | None = ...,
        expandable: global___Block.Expandable | None = ...,
        form: global___Block.Form | None = ...,
        tab_container: global___Block.TabContainer | None = ...,
        tab: global___Block.Tab | None = ...,
        chat_message: global___Block.ChatMessage | None = ...,
        popover: global___Block.Popover | None = ...,
        dialog: global___Block.Dialog | None = ...,
        flex_container: global___Block.FlexContainer | None = ...,
        allow_empty: builtins.bool = ...,
        id: builtins.str | None = ...,
        height_config: streamlit.proto.HeightConfig_pb2.HeightConfig | None = ...,
        width_config: streamlit.proto.WidthConfig_pb2.WidthConfig | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_height_config", b"_height_config", "_id", b"_id", "_width_config", b"_width_config", "chat_message", b"chat_message", "column", b"column", "dialog", b"dialog", "expandable", b"expandable", "flex_container", b"flex_container", "form", b"form", "height_config", b"height_config", "horizontal", b"horizontal", "id", b"id", "popover", b"popover", "tab", b"tab", "tab_container", b"tab_container", "type", b"type", "vertical", b"vertical", "width_config", b"width_config"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_height_config", b"_height_config", "_id", b"_id", "_width_config", b"_width_config", "allow_empty", b"allow_empty", "chat_message", b"chat_message", "column", b"column", "dialog", b"dialog", "expandable", b"expandable", "flex_container", b"flex_container", "form", b"form", "height_config", b"height_config", "horizontal", b"horizontal", "id", b"id", "popover", b"popover", "tab", b"tab", "tab_container", b"tab_container", "type", b"type", "vertical", b"vertical", "width_config", b"width_config"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_height_config", b"_height_config"]) -> typing.Literal["height_config"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_id", b"_id"]) -> typing.Literal["id"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_width_config", b"_width_config"]) -> typing.Literal["width_config"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["type", b"type"]) -> typing.Literal["vertical", "horizontal", "column", "expandable", "form", "tab_container", "tab", "chat_message", "popover", "dialog", "flex_container"] | None: ...

global___Block = Block



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: streamlit/proto/Arrow.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bstreamlit/proto/Arrow.proto\"\xd1\x04\n\x05\x41rrow\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\x17\n\x06styler\x18\x02 \x01(\x0b\x32\x07.Styler\x12\x11\n\x05width\x18\x03 \x01(\rB\x02\x18\x01\x12\x12\n\x06height\x18\x04 \x01(\rB\x02\x18\x01\x12\x1f\n\x13use_container_width\x18\x05 \x01(\x08\x42\x02\x18\x01\x12\n\n\x02id\x18\x06 \x01(\t\x12\x0f\n\x07\x63olumns\x18\x07 \x01(\t\x12(\n\x0c\x65\x64iting_mode\x18\x08 \x01(\x0e\x32\x12.Arrow.EditingMode\x12\x10\n\x08\x64isabled\x18\t \x01(\x08\x12\x0f\n\x07\x66orm_id\x18\n \x01(\t\x12\x14\n\x0c\x63olumn_order\x18\x0b \x03(\t\x12,\n\x0eselection_mode\x18\x0c \x03(\x0e\x32\x14.Arrow.SelectionMode\x12\x17\n\nrow_height\x18\r \x01(\rH\x00\x88\x01\x01\x12&\n\x0b\x62order_mode\x18\x0e \x01(\x0e\x32\x11.Arrow.BorderMode\"4\n\x0b\x45\x64itingMode\x12\r\n\tREAD_ONLY\x10\x00\x12\t\n\x05\x46IXED\x10\x01\x12\x0b\n\x07\x44YNAMIC\x10\x02\"t\n\rSelectionMode\x12\x0e\n\nSINGLE_ROW\x10\x00\x12\r\n\tMULTI_ROW\x10\x01\x12\x11\n\rSINGLE_COLUMN\x10\x02\x12\x10\n\x0cMULTI_COLUMN\x10\x03\x12\x0f\n\x0bSINGLE_CELL\x10\x04\x12\x0e\n\nMULTI_CELL\x10\x05\"/\n\nBorderMode\x12\x07\n\x03\x41LL\x10\x00\x12\x08\n\x04NONE\x10\x01\x12\x0e\n\nHORIZONTAL\x10\x02\x42\r\n\x0b_row_height\"O\n\x06Styler\x12\x0c\n\x04uuid\x18\x01 \x01(\t\x12\x0f\n\x07\x63\x61ption\x18\x02 \x01(\t\x12\x0e\n\x06styles\x18\x03 \x01(\t\x12\x16\n\x0e\x64isplay_values\x18\x04 \x01(\x0c\x42*\n\x1c\x63om.snowflake.apps.streamlitB\nArrowProtob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'streamlit.proto.Arrow_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.snowflake.apps.streamlitB\nArrowProto'
  _globals['_ARROW'].fields_by_name['width']._loaded_options = None
  _globals['_ARROW'].fields_by_name['width']._serialized_options = b'\030\001'
  _globals['_ARROW'].fields_by_name['height']._loaded_options = None
  _globals['_ARROW'].fields_by_name['height']._serialized_options = b'\030\001'
  _globals['_ARROW'].fields_by_name['use_container_width']._loaded_options = None
  _globals['_ARROW'].fields_by_name['use_container_width']._serialized_options = b'\030\001'
  _globals['_ARROW']._serialized_start=32
  _globals['_ARROW']._serialized_end=625
  _globals['_ARROW_EDITINGMODE']._serialized_start=391
  _globals['_ARROW_EDITINGMODE']._serialized_end=443
  _globals['_ARROW_SELECTIONMODE']._serialized_start=445
  _globals['_ARROW_SELECTIONMODE']._serialized_end=561
  _globals['_ARROW_BORDERMODE']._serialized_start=563
  _globals['_ARROW_BORDERMODE']._serialized_end=610
  _globals['_STYLER']._serialized_start=627
  _globals['_STYLER']._serialized_end=706
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: streamlit/proto/Alert.proto
# Protobuf Python Version: 5.26.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from streamlit.proto import WidthConfig_pb2 as streamlit_dot_proto_dot_WidthConfig__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1bstreamlit/proto/Alert.proto\x1a!streamlit/proto/WidthConfig.proto\"\xb5\x01\n\x05\x41lert\x12\x0c\n\x04\x62ody\x18\x01 \x01(\t\x12\x1d\n\x06\x66ormat\x18\x02 \x01(\x0e\x32\r.Alert.Format\x12\x0c\n\x04icon\x18\x03 \x01(\t\x12,\n\x0cwidth_config\x18\x04 \x01(\x0b\x32\x16.streamlit.WidthConfig\"C\n\x06\x46ormat\x12\n\n\x06UNUSED\x10\x00\x12\t\n\x05\x45RROR\x10\x01\x12\x0b\n\x07WARNING\x10\x02\x12\x08\n\x04INFO\x10\x03\x12\x0b\n\x07SUCCESS\x10\x04\x42*\n\x1c\x63om.snowflake.apps.streamlitB\nAlertProtob\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'streamlit.proto.Alert_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\034com.snowflake.apps.streamlitB\nAlertProto'
  _globals['_ALERT']._serialized_start=67
  _globals['_ALERT']._serialized_end=248
  _globals['_ALERT_FORMAT']._serialized_start=181
  _globals['_ALERT_FORMAT']._serialized_end=248
# @@protoc_insertion_point(module_scope)



# Copyright (c) Streamlit Inc. (2018-2022) Snowflake Inc. (2022-2025)
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import annotations

import copy
import os
import re
import urllib.error
import urllib.request
from typing import TYPE_CHECKING, Any

if TYPE_CHECKING:
    from collections.abc import Iterator

from streamlit import cli_util, url_util
from streamlit.config_option import ConfigOption
from streamlit.elements.lib.color_util import is_css_color_like
from streamlit.errors import StreamlitAPIException

# Maximum size for theme files (1MB). Theme files should be small configuration
# files containing only theme options, not large data files.
_MAX_THEME_FILE_SIZE_BYTES = 1024 * 1024  # 1MB


def _get_logger() -> Any:
    """Get logger for this module. Separate function to avoid circular imports."""
    from streamlit.logger import get_logger

    return get_logger(__name__)


def server_option_changed(
    old_options: dict[str, ConfigOption], new_options: dict[str, ConfigOption]
) -> bool:
    """Return True if and only if an option in the server section differs
    between old_options and new_options.
    """
    for opt_name, opt_val in old_options.items():
        if not opt_name.startswith("server"):
            continue

        old_val = opt_val.value
        new_val = new_options[opt_name].value
        if old_val != new_val:
            return True

    return False


def show_config(
    section_descriptions: dict[str, str],
    config_options: dict[str, ConfigOption],
) -> None:
    """Print the given config sections/options to the terminal."""

    out = []
    out.append(
        _clean(
            """
        # Below are all the sections and options you can have in
        ~/.streamlit/config.toml.
    """
        )
    )

    def append_desc(text: str) -> None:
        out.append("# " + cli_util.style_for_cli(text, bold=True))

    def append_comment(text: str) -> None:
        out.append("# " + cli_util.style_for_cli(text))

    def append_section(text: str) -> None:
        out.append(cli_util.style_for_cli(text, bold=True, fg="green"))

    def append_setting(text: str) -> None:
        out.append(cli_util.style_for_cli(text, fg="green"))

    for section in section_descriptions:
        # We inject a fake config section used for unit tests that we exclude here as
        # its options are often missing required properties, which confuses the code
        # below.
        if section == "_test":
            continue

        section_options = {
            k: v
            for k, v in config_options.items()
            if v.section == section and v.visibility == "visible" and not v.is_expired()
        }

        # Only show config header if section is non-empty.
        if len(section_options) == 0:
            continue

        out.append("")
        append_section(f"[{section}]")
        out.append("")

        for option in section_options.values():
            key = option.key.split(".")[-1]
            description_paragraphs = _clean_paragraphs(option.description or "")

            last_paragraph_idx = len(description_paragraphs) - 1

            for i, paragraph in enumerate(description_paragraphs):
                # Split paragraph into lines
                lines = paragraph.rstrip().split(
                    "\n"
                )  # Remove trailing newline characters

                # If the first line is empty, remove it
                if lines and not lines[0].strip():
                    lines = lines[1:]

                # Choose function based on whether it's the first paragraph or not
                append_func = append_desc if i == 0 else append_comment

                # Add comment character to each line and add to out
                for line in lines:
                    append_func(line.lstrip())

                # # Add a line break after a paragraph only if it's not the last paragraph
                if i != last_paragraph_idx:
                    append_comment("")

            if option.deprecated:
                if out[-1] != "#":
                    append_comment("")
                append_comment(
                    cli_util.style_for_cli("THIS IS DEPRECATED.", fg="yellow")
                )
                append_comment("")
                for line in _clean_paragraphs(option.deprecation_text):
                    append_comment(line)
                append_comment("")
                append_comment(
                    f"This option will be removed on or after {option.expiration_date}."
                )

            import toml

            toml_default = toml.dumps({"default": option.default_val})
            toml_default = toml_default[10:].strip()

            if len(toml_default) > 0:
                # Ensure a line break before appending "Default" comment, if not already there
                if out[-1] != "#":
                    append_comment("")
                append_comment(f"Default: {toml_default}")
            else:
                # Don't say "Default: (unset)" here because this branch applies
                # to complex config settings too.
                pass

            option_is_manually_set = (
                option.where_defined != ConfigOption.DEFAULT_DEFINITION
            )

            if option_is_manually_set:
                if out[-1] != "# ":
                    append_comment("")
                append_comment(f"The value below was set in {option.where_defined}")

            toml_setting = toml.dumps({key: option.value})

            if len(toml_setting) == 0:
                toml_setting = f"# {key} =\n"
            elif not option_is_manually_set:
                toml_setting = f"# {toml_setting}"

            append_setting(toml_setting)

    cli_util.print_to_cli("\n".join(out))


def _clean(txt: str) -> str:
    """Replace sequences of multiple spaces with a single space, excluding newlines.

    Preserves leading and trailing spaces, and does not modify spaces in between lines.
    """
    return re.sub(" +", " ", txt)


def _clean_paragraphs(txt: str) -> list[str]:
    """Split the text into paragraphs, preserve newlines within the paragraphs."""
    # Strip both leading and trailing newlines.
    txt = txt.strip("\n")
    paragraphs = txt.split("\n\n")
    return [
        "\n".join(_clean(line) for line in paragraph.split("\n"))
        for paragraph in paragraphs
    ]


# Theme configuration - theme.base support functions


def _check_color_value(value: Any, option_name: str) -> None:
    """
    Lightweight check of theme color config option values.

    Validates that the value is a string (or list of strings, in the case of
    chartCategoricalColors and chartSequentialColors) and is not empty.
    Logs warnings for potentially invalid colors, since we do more comprehensive validation on the frontend.

    Handles both single color strings (like primaryColor, backgroundColor)
    and arrays of color strings (like chartCategoricalColors, chartSequentialColors).

    Raises StreamlitAPIException for type errors or empty values.
    Does not return anything - used purely for validation side effects.
    """
    logger = _get_logger()

    # Handle array color options (chartCategoricalColors, chartSequentialColors)
    if isinstance(value, list):
        if not value:
            raise StreamlitAPIException(
                f"Theme option '{option_name}' cannot be an empty array"
            )

        for i, color in enumerate(value):
            if not isinstance(color, str):
                raise StreamlitAPIException(
                    f"Theme option '{option_name}[{i}]' must be a string, got {type(color).__name__}: {color}"
                )

            color_str = color.strip()
            if not color_str:
                raise StreamlitAPIException(
                    f"Theme option '{option_name}[{i}]' cannot be empty"
                )

            # Lightweight color validation with warning
            if not is_css_color_like(color_str):
                logger.warning(
                    "Theme option '%s[%s]' may be an invalid color: %s. "
                    "Expected formats: hex, rgb, and rgba colors",
                    option_name,
                    i,
                    color_str,
                )

        return  # All colors in array have been checked

    # Handle single color options (primaryColor, backgroundColor, etc.)
    if not isinstance(value, str):
        raise StreamlitAPIException(
            f"Theme option '{option_name}' must be a string or array of strings, got {type(value).__name__}: {value}"
        )

    value_str: str = value.strip()

    if not value_str:
        raise StreamlitAPIException(f"Theme option '{option_name}' cannot be empty")

    # Lightweight color validation with warning
    if not is_css_color_like(value_str):
        logger.warning(
            "Theme option '%s' may be an invalid color: %s. "
            "Expected formats: hex, rgb, and rgba colors",
            option_name,
            value_str,
        )


def _iterate_theme_config_options(
    config_options: dict[str, ConfigOption],
) -> Iterator[tuple[str, Any]]:
    """
    Iterate through theme config options, yielding (option_path, value) pairs.
    Returns: theme.primaryColor, #ff0000, ...
    """
    for opt_name, opt_val in config_options.items():
        if opt_name.startswith("theme.") and opt_val.value is not None:
            yield opt_name, opt_val.value


def _extract_current_theme_config(
    config_options: dict[str, ConfigOption],
) -> dict[str, Any]:
    """
    Extract current theme configuration from config options.
    Returns a dictionary with the current theme options in nested format.
    """
    current_theme_options = {}

    for opt_name, opt_value in _iterate_theme_config_options(config_options):
        parts = opt_name.split(".")
        if len(parts) == 2:  # theme.option
            _, option = parts
            if option != "base":  # Don't include the base option itself
                current_theme_options[option] = opt_value
        elif len(parts) == 3:  # theme.sidebar.option
            _, section, option = parts
            if section not in current_theme_options:
                current_theme_options[section] = {}
            current_theme_options[section][option] = opt_value

    return current_theme_options


def _get_valid_theme_options(
    config_options_template: dict[str, ConfigOption], section: str = "main"
) -> set[str]:
    """Get the set of valid theme configuration options for a specific section.

    Extracts valid theme options from the config options template to ensure they
    stay in sync with the actual theme options defined via _create_theme_options() calls.

    Returns a set of valid theme option names for a specific section (without the "theme." prefix)
    """
    valid_options = set()

    # Extract theme options from the config template
    for option_key in config_options_template:
        if option_key.startswith("theme."):
            parts = option_key.split(".")
            if section == "main" and len(parts) == 2:
                # Direct theme option like "theme.primaryColor"
                _, option_name = parts
                valid_options.add(option_name)
            elif section == "sidebar" and len(parts) == 3 and parts[1] == "sidebar":
                # Sidebar option like "theme.sidebar.primaryColor"
                _, _, option_name = parts
                valid_options.add(option_name)

    return valid_options


def _invalid_theme_option_warning(
    option_name: str,
    file_path_or_url: str,
    valid_options: set[str],
    section_name: str = "theme",
) -> None:
    """Log a warning for an invalid theme option."""

    full_option_name = (
        f"{section_name}.{option_name}"
        if section_name == "theme"
        else f"theme.{section_name}.{option_name}"
    )

    valid_options_list = "\n".join(f"  • {opt}" for opt in sorted(valid_options))
    _get_logger().warning(
        "Theme file %s contains invalid theme option: '%s'.\n\n"
        "Valid '%s' options are:\n%s",
        file_path_or_url,
        full_option_name,
        section_name,
        valid_options_list,
    )


def _validate_theme_file_content(
    theme_content: dict[str, Any],
    file_path_or_url: str,
    config_options_template: dict[str, ConfigOption],
) -> dict[str, Any]:
    """
    Validate that a theme file contains only valid theme sections and config options.

    If invalid sections are found in the theme file, a StreamlitAPIException is raised.

    If invalid config options are found in the theme file, a warning is logged with the valid
    options for the given section.

    Returns
    -------
        A filtered copy of the theme content with invalid options removed.
    """
    valid_main_options = _get_valid_theme_options(config_options_template, "main")
    valid_subsections = {"sidebar"}

    theme_section = theme_content.get("theme", {})

    # Create a filtered copy of the theme content
    filtered_theme = copy.deepcopy(theme_content)
    filtered_theme_section = filtered_theme.get("theme", {})

    # Validate theme options
    for option_name, option_value in theme_section.items():
        # Handle subsection (valid subsection is only sidebar for now)
        if isinstance(option_value, dict):
            # Invalid subsection: raise error
            if option_name not in valid_subsections:
                raise StreamlitAPIException(
                    f"Theme file {file_path_or_url} contains invalid theme subsection: '{option_name}'.\n\n"
                    f"Valid subsections are: {valid_subsections}"
                )

            # Rename option_name for clarity - it's actually the section name in this case (i.e. sidebar)
            section_name = option_name

            # Get valid options for this specific subsection
            valid_section_options = _get_valid_theme_options(
                config_options_template, section_name
            )

            # Validate options within the subsection
            for section_option, section_value in option_value.items():
                if section_option not in valid_section_options:
                    _invalid_theme_option_warning(
                        section_option,
                        file_path_or_url,
                        valid_section_options,
                        section_name,
                    )
                    # Remove invalid option from filtered theme
                    if (
                        section_name in filtered_theme_section
                        and section_option in filtered_theme_section[section_name]
                    ):
                        del filtered_theme_section[section_name][section_option]
                # Check color options for subsection
                full_option_name = f"theme.{section_name}.{section_option}"
                if "color" in full_option_name.lower():
                    _check_color_value(section_value, full_option_name)

        # Handle main theme options
        else:
            if option_name not in valid_main_options:
                _invalid_theme_option_warning(
                    option_name,
                    file_path_or_url,
                    valid_main_options,
                )
                # Remove invalid option from filtered theme
                if option_name in filtered_theme_section:
                    del filtered_theme_section[option_name]
            # Check color options for direct theme option
            full_option_name = f"theme.{option_name}"
            if "color" in full_option_name.lower():
                _check_color_value(option_value, full_option_name)

    return filtered_theme


def _load_theme_file(
    file_path_or_url: str, config_options_template: dict[str, ConfigOption]
) -> dict[str, Any]:
    """
    Load and parse a theme TOML file from a local path or URL.

    Handles raising errors when a file cannot be found, read, parsed,
    or contains invalid theme options.

    Otherwise returns the parsed TOML content as a dictionary.
    """

    def _raise_missing_toml() -> None:
        raise StreamlitAPIException(
            "The 'toml' package is required to load theme files. "
            "Please install it with 'pip install toml'."
        )

    def _raise_file_not_found() -> None:
        raise FileNotFoundError(f"Theme file not found: {file_path_or_url}")

    def _raise_missing_theme_section() -> None:
        raise StreamlitAPIException(
            f"Theme file {file_path_or_url} must contain a [theme] section"
        )

    def _raise_file_too_large() -> None:
        content_size = len(content.encode("utf-8"))
        raise StreamlitAPIException(
            f"Theme file {file_path_or_url} is too large ({content_size:,} bytes). "
            f"Maximum allowed size is {_MAX_THEME_FILE_SIZE_BYTES:,} bytes (1MB). "
            f"Theme files should contain only configuration options, not large data."
        )

    try:
        import toml
    except ImportError:
        _raise_missing_toml()

    # Check if it's a URL using the url_util helper (only allow http/https schemes by default)
    is_valid_url = url_util.is_url(file_path_or_url)

    try:
        if is_valid_url:
            # Load from URL - noqa: S310 suppressed since url_util.is_url() restricts to only
            # http/https schemes by default, preventing file:// or other dangerous schemes
            # 30-second timeout prevents hanging in poor network conditions (same as cli.py)
            with urllib.request.urlopen(file_path_or_url, timeout=30) as response:  # noqa: S310
                content = response.read().decode("utf-8")
        else:
            # Load from local file path
            # Resolve relative paths from the current working directory
            if not os.path.isabs(file_path_or_url):
                file_path_or_url = os.path.join(os.getcwd(), file_path_or_url)

            if not os.path.exists(file_path_or_url):
                _raise_file_not_found()

            with open(file_path_or_url, encoding="utf-8") as f:
                content = f.read()

        # Check file size limit - theme files should be small configuration files
        content_size = len(content.encode("utf-8"))
        if content_size > _MAX_THEME_FILE_SIZE_BYTES:
            _raise_file_too_large()

        # Parse the TOML content
        parsed_theme = toml.loads(content)

        # Validate that the theme file has a theme section
        if "theme" not in parsed_theme:
            _raise_missing_theme_section()

        # Validate that the theme file contains only valid theme options, filtering out invalid ones
        filtered_theme = _validate_theme_file_content(
            parsed_theme, file_path_or_url, config_options_template
        )

        return filtered_theme

    except (StreamlitAPIException, FileNotFoundError):
        # Re-raise these specific exceptions
        raise
    except urllib.error.URLError as e:
        raise StreamlitAPIException(
            f"Could not load theme file from URL {file_path_or_url}: {e}"
        ) from e
    except Exception as e:
        raise StreamlitAPIException(
            f"Error loading theme file {file_path_or_url}: {e}"
        ) from e


def _apply_theme_inheritance(
    base_theme: dict[str, Any], override_theme: dict[str, Any]
) -> dict[str, Any]:
    """
    Apply theme inheritance where theme config values from config.toml
    take precedence over the theme configs defined in theme.base toml file.

    Returns a dictionary with the merged theme configuration.
    """
    # Start with a deep copy of the base theme
    merged_theme = copy.deepcopy(base_theme)

    # Apply overrides from the config.toml
    for section_name, section_data in override_theme.items():
        if section_name not in merged_theme:
            merged_theme[section_name] = {}

        if isinstance(section_data, dict):
            # Handle nested sections like theme.sidebar
            for key, value in section_data.items():
                if (
                    isinstance(value, dict)
                    and key in merged_theme[section_name]
                    and isinstance(merged_theme[section_name][key], dict)
                ):
                    # Merge nested dictionaries (e.g., theme.sidebar)
                    merged_theme[section_name][key].update(value)
                else:
                    # Direct assignment for non-dict values or new keys
                    merged_theme[section_name][key] = value
        else:
            # Handle direct theme options
            merged_theme[section_name] = section_data

    return merged_theme


# Theme configuration - handles theme.base


def process_theme_inheritance(
    config_options: dict[str, ConfigOption] | None,
    config_options_template: dict[str, ConfigOption],
    set_option_func: Any,
) -> None:
    """
    Process theme inheritance if theme.base points to a theme file.

    This function checks if theme.base is set to a file path or URL,
    loads the theme file, and applies inheritance logic where the
    current config.toml values override the theme.base file values.

    Sets the merged theme options to the config.
    """
    # Get the current theme.base value
    if config_options is None:
        return

    base_option = config_options.get("theme.base")
    if not base_option or base_option.value is None:
        return

    base_value = base_option.value

    # Check if it's a file path or URL (not just "light" or "dark")
    if base_value in ("light", "dark"):
        return

    def _raise_invalid_nested_base() -> None:
        raise StreamlitAPIException(
            f"Theme file {base_value} cannot reference another theme file in its base property. "
            f"Only 'light' and 'dark' are allowed in referenced theme files."
        )

    try:
        # Load the theme file config options
        theme_file_content = _load_theme_file(base_value, config_options_template)

        # Validate that theme.base of the referenced theme file doesn't reference another file
        theme_base = theme_file_content.get("theme", {}).get("base")
        if theme_base and theme_base not in ("light", "dark"):
            _raise_invalid_nested_base()

        # Get current theme options from main config.toml
        current_theme_options = (
            _extract_current_theme_config(config_options) if config_options else {}
        )

        # Apply inheritance: referenced theme file as base, override with theme options specified in config.toml
        merged_theme = _apply_theme_inheritance(
            theme_file_content, {"theme": current_theme_options}
        )

        # Preserve theme options set by env vars and command line flags (higher precedence)
        high_precedence_theme_options = {}
        if config_options is not None:
            for opt_name, opt_config in config_options.items():
                if (
                    opt_name.startswith("theme.")
                    and opt_name != "theme.base"
                    and opt_config.where_defined
                    in (
                        "environment variable",
                        "command-line argument or environment variable",
                    )
                ):
                    high_precedence_theme_options[opt_name] = {
                        "value": opt_config.value,
                        "where_defined": opt_config.where_defined,
                    }

            # Clear existing theme options (except base) to prepare for inheritance
            theme_options_to_remove = [
                opt_name
                for opt_name in config_options
                if opt_name.startswith("theme.") and opt_name != "theme.base"
            ]
            for opt_name in theme_options_to_remove:
                set_option_func(opt_name, None, "reset for theme inheritance")

        # Handle theme.base - always set it to a valid value ("light" or "dark", not a path/URL)
        theme_file_base = theme_file_content.get("theme", {}).get("base")
        if theme_file_base:
            set_option_func("theme.base", theme_file_base, f"theme file: {base_value}")
        else:
            # Theme file doesn't specify a base, default to "light"
            set_option_func(
                "theme.base", "light", f"theme file: {base_value} (default)"
            )

        # Set the merged theme options
        theme_section = merged_theme.get("theme", {})

        for option_name, option_value in theme_section.items():
            if option_name == "base":
                # Already handled above
                continue

            if isinstance(option_value, dict):
                # Handle nested sections like sidebar
                for sub_option, sub_value in option_value.items():
                    set_option_func(
                        f"theme.{option_name}.{sub_option}",
                        sub_value,
                        f"theme file: {base_value}",
                    )
            else:
                set_option_func(
                    f"theme.{option_name}", option_value, f"theme file: {base_value}"
                )

        # Finally, restore theme options set by env vars and command line flags (highest precedence)
        for opt_name, opt_data in high_precedence_theme_options.items():
            set_option_func(opt_name, opt_data["value"], opt_data["where_defined"])

    except (StreamlitAPIException, FileNotFoundError):
        # Re-raise expected user errors as-is to preserve specific error messages
        raise
    except Exception as e:
        _get_logger().exception("Error processing theme inheritance")
        # Only wrap unexpected errors (not our specific validation errors)
        raise StreamlitAPIException(
            f"Failed to process theme inheritance from {base_value}: {e}"
        ) from e
